import sys
from hashlib import md5

from django.conf import settings
from django.core.cache import DEFAULT_CACHE_ALIAS
from user_agents import parse


# `get_cache` function has been deprecated since Django 1.7 in favor of `caches`.
try:
    from django.core.cache import caches

    def get_cache(backend, **kwargs):
        return caches[backend]
except ImportError:
    from django.core.cache import get_cache


# Small snippet from the `six` library to help with Python 3 compatibility
if sys.version_info[0] == 3:
    text_type = str
else:
    text_type = unicode


USER_AGENTS_CACHE = getattr(settings, 'USER_AGENTS_CACHE', DEFAULT_CACHE_ALIAS)

if USER_AGENTS_CACHE:
    cache = get_cache(USER_AGENTS_CACHE)
else:
    cache = None


def get_cache_key(ua_string):
    # Some user agent strings are longer than 250 characters so we use its MD5
    if isinstance(ua_string, text_type):
        ua_string = ua_string.encode('utf-8')
    return ''.join(['django_user_agents.', md5(ua_string).hexdigest()])


def get_user_agent(request):
    # Tries to get UserAgent objects from cache before constructing a UserAgent
    # from scratch because parsing regexes.yaml/json (ua-parser) is slow
    if not hasattr(request, 'META'):
        return ''

    ua_string = request.META.get('HTTP_USER_AGENT', '')

    if not isinstance(ua_string, text_type):
        ua_string = ua_string.decode('utf-8', 'ignore')

    if cache:
        key = get_cache_key(ua_string)
        user_agent = cache.get(key)
        if user_agent is None:
            user_agent = parse(ua_string)
            cache.set(key, user_agent)
    else:
        user_agent = parse(ua_string)
    return user_agent


def get_and_set_user_agent(request):
    # If request already has ``user_agent``, it will return that, otherwise
    # call get_user_agent and attach it to request so it can be reused
    if hasattr(request, 'user_agent'):
        return request.user_agent

    if not request:
        return parse('')

    request.user_agent = get_user_agent(request)
    return request.user_agent



import django
from os import path


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    },
}

INSTALLED_APPS = ['django_user_agents']

if django.VERSION >= (1, 10):
    MIDDLEWARE = [
        'django_user_agents.middleware.UserAgentMiddleware',
    ]
else:
    MIDDLEWARE_CLASSES = [
        'django_user_agents.middleware.UserAgentMiddleware',
    ]

ROOT_URLCONF = 'django_user_agents.tests.urls'

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'TIMEOUT': 60,
        'LOCATION': 'default-location',
    },
    'test': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'TIMEOUT': 60,
        'LOCATION': 'test-location',
    },
}

if django.VERSION >= (1, 8):
    TEMPLATES = [
        {
            'BACKEND': 'django.template.backends.django.DjangoTemplates',
            'DIRS': [
                path.join(path.dirname(__file__), "templates"),
            ],
        },
    ]
else:
    TEMPLATE_DIRS = (
        path.join(path.dirname(__file__), "templates"),
    )

SECRET_KEY = 'foobarbaz'



import django
from django.conf.urls import url
from django_user_agents.tests import views


if django.VERSION >= (1, 8):
    urlpatterns = [
        url(r'^user-agents/', views.test, name='user_agent_test'),
        url(r'^filters/', views.test_filters, name='user_agent_test_filters'),
    ]
else:
    from django.conf.urls import patterns
    urlpatterns = patterns(
        'django_user_agents.tests.views',
        url(r'^user-agents/', 'test', name='user_agent_test'),
        url(r'^filters/', 'test_filters', name='user_agent_test_filters'),
    )



from django import template

from ..utils import get_and_set_user_agent


register = template.Library()


@register.filter()
def is_mobile(request):
    return get_and_set_user_agent(request).is_mobile


@register.filter()
def is_pc(request):
    return get_and_set_user_agent(request).is_pc


@register.filter()
def is_tablet(request):
    return get_and_set_user_agent(request).is_tablet


@register.filter()
def is_bot(request):
    return get_and_set_user_agent(request).is_bot


@register.filter()
def is_touch_capable(request):
    return get_and_set_user_agent(request).is_touch_capable



# coding=utf-8
from django.core.cache import cache
from django.test.client import Client, RequestFactory
from django.test.utils import override_settings
from django.test import SimpleTestCase

from user_agents.parsers import UserAgent
from django_user_agents import utils
from django_user_agents.utils import get_cache_key, get_user_agent, get_and_set_user_agent
from django_user_agents.templatetags import user_agents

try:
    # The reload() function has been moved from imp to importlib since python 3.4
    from importlib import reload as reload_module
except ImportError:
    try:
        from imp import reload as reload_module
    except ImportError:
        reload_module = reload  # python 2.x

try:
    from django.urls import reverse  # django 1.10+
except ImportError:
    from django.core.urlresolvers import reverse


iphone_ua_string = 'Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B179 Safari/7534.48.3'
ipad_ua_string = 'Mozilla/5.0(iPad; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B314 Safari/531.21.10'
long_ua_string = 'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.3; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)'


class MiddlewareTest(SimpleTestCase):

    def tearDown(self):
        for ua in [iphone_ua_string, ipad_ua_string, long_ua_string]:
            cache.delete(get_cache_key(ua))

    def test_middleware_assigns_user_agent(self):
        client = Client(HTTP_USER_AGENT=ipad_ua_string)
        response = client.get(reverse('user_agent_test'))
        self.assertIsInstance(response.context['user_agent'], UserAgent)

    def test_cache_is_set(self):
        request = RequestFactory(HTTP_USER_AGENT=iphone_ua_string).get('')
        user_agent = get_user_agent(request)
        self.assertIsInstance(user_agent, UserAgent)
        self.assertIsInstance(cache.get(get_cache_key(iphone_ua_string)), UserAgent)

    def test_empty_user_agent_does_not_cause_error(self):
        request = RequestFactory().get('')
        user_agent = get_user_agent(request)
        self.assertIsInstance(user_agent, UserAgent)

    def test_get_and_set_user_agent(self):
        # Test that get_and_set_user_agent attaches ``user_agent`` to request
        request = RequestFactory().get('')
        get_and_set_user_agent(request)
        self.assertIsInstance(request.user_agent, UserAgent)

    def test_filters_can_be_loaded_in_template(self):
        client = Client(HTTP_USER_AGENT=ipad_ua_string)
        response = client.get(reverse('user_agent_test_filters'))
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            'Just making sure all the filters can be used without errors')

    def test_filters(self):
        request = RequestFactory(HTTP_USER_AGENT=iphone_ua_string).get('')
        self.assertTrue(user_agents.is_mobile(request))
        self.assertTrue(user_agents.is_touch_capable(request))
        self.assertFalse(user_agents.is_tablet(request))
        self.assertFalse(user_agents.is_pc(request))
        self.assertFalse(user_agents.is_bot(request))

    def test_get_cache_key(self):
        self.assertEqual(
            get_cache_key(long_ua_string),
            'django_user_agents.c226ec488bae76c60dd68ad58f03d729',
        )
        self.assertEqual(
            get_cache_key(iphone_ua_string),
            'django_user_agents.00705b9375a0e46e966515fe90f111da',
        )

    @override_settings(USER_AGENTS_CACHE=None)
    def test_disabled_cache(self):
        reload_module(utils)  # re-import with patched settings

        request = RequestFactory(HTTP_USER_AGENT=iphone_ua_string).get('')
        user_agent = get_user_agent(request)
        self.assertIsInstance(user_agent, UserAgent)
        self.assertIsNone(cache.get(get_cache_key(iphone_ua_string)))

    @override_settings(USER_AGENTS_CACHE='test')
    def test_custom_cache(self):
        reload_module(utils)  # re-import with patched settings

        request = RequestFactory(HTTP_USER_AGENT=iphone_ua_string).get('')
        user_agent = get_user_agent(request)
        self.assertIsInstance(user_agent, UserAgent)
        self.assertIsNone(cache.get(get_cache_key(iphone_ua_string)))
        self.assertIsInstance(utils.cache.get(get_cache_key(iphone_ua_string)), UserAgent)


unicode_ua_string = 'Mozilla/5.0 (Linux; Android 4.4.2; X325 – Locked to Life Wireless Build/KOT49H) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/30.0.0.0 Mobile Safari/537.36'

class UtilsTest(SimpleTestCase):

    @override_settings(USER_AGENTS_CACHE=None)
    def test_unicode_ua_string(self):
        reload_module(utils)  # re-import with patched settings

        request = RequestFactory(HTTP_USER_AGENT=unicode_ua_string).get('')
        
        self.assertTrue(user_agents.is_mobile(request))



from django.shortcuts import render


def test(request):
    return render(request, "test.html", {'user_agent': request.user_agent})


def test_filters(request):
    return render(request, "test_filters.html", {'request': request})



# -*- coding: utf-8 -*-
from setuptools import setup


description = ("A django package that allows easy identification of visitors' "
               "browser, operating system and device information (mobile "
               "phone, tablet or has touch capabilities).")

setup(
    name='django-user_agents',
    version='0.4.0',
    author='Selwin Ong',
    author_email='selwin.ong@gmail.com',
    packages=['django_user_agents'],
    url='https://github.com/selwin/django-user_agents',
    license='MIT',
    description=description,
    long_description=open('README.rst').read(),
    zip_safe=False,
    include_package_data=True,
    package_data={'': ['README.rst']},
    install_requires=['django', 'user-agents'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



from django.utils.functional import SimpleLazyObject

from .utils import get_user_agent


class UserAgentMiddleware(object):

    def __init__(self, get_response=None):
        if get_response is not None:
            self.get_response = get_response

    def __call__(self, request):
        self.process_request(request)
        return self.get_response(request)

    def process_request(self, request):
        request.user_agent = SimpleLazyObject(lambda: get_user_agent(request))
